<?php

namespace Crm\School\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Crm\School\Forms\Gallery as GalleryForm;
use Crm\School\Http\Requests\GalleryResourceRequest;
use Crm\School\Http\Resources\GalleryResource;
use Crm\School\Http\Resources\GalleriesCollection;
use Crm\School\Models\Gallery;
use Crm\School\Actions\GalleryAction;
use Crm\School\Actions\GalleryActions;


/**
 * Resource controller class for gallery.
 */
class GalleryResourceController extends BaseController
{

    /**
     * Initialize gallery resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = GalleryForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('crm.school.modules'), 'school', guard_url('school'));
            return $next($request);
        });
    }

    /**
     * Display a list of gallery.
     *
     * @return Response
     */
    public function index(GalleryResourceRequest $request)
    {
        $request = $request->all();
        $page = GalleryActions::run('paginate', $request);

        $data = new GalleriesCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('school::gallery.names'))
            ->view('school::gallery.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display gallery.
     *
     * @param Request $request
     * @param Model   $gallery
     *
     * @return Response
     */
    public function show(GalleryResourceRequest $request, Gallery $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new GalleryResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('school::gallery.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('school::gallery.show')
            ->output();
    }

    /**
     * Show the form for creating a new gallery.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(GalleryResourceRequest $request, Gallery $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new GalleryResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('school::gallery.name'))
            ->view('school::gallery.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new gallery.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(GalleryResourceRequest $request, Gallery $model)
    {
        try {
            $request = $request->all();
            $model = GalleryAction::run('store', $model, $request);
            $data = new GalleryResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('school::gallery.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('school/gallery/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/school/gallery'))
                ->redirect();
        }

    }

    /**
     * Show gallery for editing.
     *
     * @param Request $request
     * @param Model   $gallery
     *
     * @return Response
     */
    public function edit(GalleryResourceRequest $request, Gallery $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new GalleryResource($model);
        // return view('school::gallery.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('school::gallery.name'))
            ->view('school::gallery.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the gallery.
     *
     * @param Request $request
     * @param Model   $gallery
     *
     * @return Response
     */
    public function update(GalleryResourceRequest $request, Gallery $model)
    {
        try {
            $request = $request->all();
            $model = GalleryAction::run('update', $model, $request);
            $data = new GalleryResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('school::gallery.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('school/gallery/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('school/gallery/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the gallery.
     *
     * @param Model   $gallery
     *
     * @return Response
     */
    public function destroy(GalleryResourceRequest $request, Gallery $model)
    {
        try {

            $request = $request->all();
            $model = GalleryAction::run('destroy', $model, $request);
            $data = new GalleryResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('school::gallery.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('school/gallery/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('school/gallery/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
